package com.zanxiang.game.module.manage.service.impl;

import cn.hutool.http.ContentType;
import cn.hutool.http.HttpUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.zanxiang.erp.base.ErpServer;
import com.zanxiang.erp.base.rpc.IDingTalkMsgRpc;
import com.zanxiang.game.module.base.ServerInfo;
import com.zanxiang.game.module.base.pojo.enums.PayApplicationTypeEnum;
import com.zanxiang.game.module.base.pojo.enums.StatusEnum;
import com.zanxiang.game.module.base.rpc.IWxApiServiceRpc;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.mybatis.entity.GamePayWay;
import com.zanxiang.game.module.mybatis.entity.ListenCall;
import com.zanxiang.game.module.mybatis.entity.PayApplication;
import com.zanxiang.game.module.mybatis.entity.PayBox;
import com.zanxiang.module.util.JsonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2024-01-20
 * @description : 小程序检查
 */
@Slf4j
@Service
public class AppletCheckServiceImpl implements IAppletCheckService {

    @DubboReference(providedBy = ErpServer.SERVER_DUBBO_NAME)
    private IDingTalkMsgRpc dingTalkMsgRpc;

    @DubboReference(providedBy = ServerInfo.SERVER_SDK_DUBBO_NAME)
    private IWxApiServiceRpc wxApiServiceRpc;

    @Autowired
    private IPayBoxService payBoxService;

    @Autowired
    private IGamePayWayService gamePayWayService;

    @Autowired
    private IListenCallService listenCallService;

    @Autowired
    private IPayApplicationService payApplicationService;

    @Override
    public void payApplicationCheck() {
        List<PayApplication> payApplicationList = payApplicationService.list(new LambdaQueryWrapper<PayApplication>()
                .eq(PayApplication::getStatus, StatusEnum.YES.getCode())
                .eq(PayApplication::getType, PayApplicationTypeEnum.WX_MINI_APP.getType()));
        if (CollectionUtils.isEmpty(payApplicationList)) {
            return;
        }
        payApplicationList.forEach(payApplication -> {
            String accessToken = wxApiServiceRpc.getAccessToken(payApplication.getAppId(), payApplication.getAppSecret());
            boolean checkResult = this.appletCheck(accessToken);
            if (checkResult) {
                return;
            }
            this.payApplicationCheck(payApplication);
        });
    }

    @Override
    public void payApplicationCheck(PayApplication payApplication) {
        try {
            //更新游戏支付盒子配置
            this.gamePayWayUpdate(payApplication);
        } catch (Exception e) {
            log.error("支付应用异常更新游戏支付盒子失败, appName : {}, e : {}", payApplication.getAppName(), e.getMessage());
        }
        try {
            //钉钉通知
            this.sendDingTalkMsg(payApplication);
        } catch (Exception e) {
            log.error("支付应用异常钉钉通知失败, appName : {}, e : {}", payApplication.getAppName(), e.getMessage());
        }
        try {
            //电话通知
            this.sendPhoneTalkMsg(payApplication);
        } catch (Exception e) {
            log.error("支付应用异常电话通知失败, appName : {}, e : {}", payApplication.getAppName(), e.getMessage());
        }
    }

    private void gamePayWayUpdate(PayApplication payApplication) {
        //修改支付应用状态
        payApplicationService.update(new LambdaUpdateWrapper<PayApplication>()
                .eq(PayApplication::getAppId, payApplication.getAppId())
                .set(PayApplication::getStatus, StatusEnum.NO.getCode())
                .set(PayApplication::getUpdateTime, LocalDateTime.now()));
        //查询支付应用对应的盒子
        List<PayBox> payBoxList = payBoxService.list(new LambdaQueryWrapper<PayBox>()
                .eq(PayBox::getAppId, payApplication.getAppId()));
        if (CollectionUtils.isEmpty(payBoxList)) {
            return;
        }
        //修改相关支付盒子状态
        payBoxService.update(new LambdaUpdateWrapper<PayBox>()
                .eq(PayBox::getAppId, payApplication.getAppId())
                .set(PayBox::getStatus, StatusEnum.NO.getCode())
                .set(PayBox::getUpdateTime, LocalDateTime.now()));
        //获取一个能用的支付盒子
        PayBox payBox = payBoxService.getOne(new LambdaQueryWrapper<PayBox>()
                .eq(PayBox::getType, payApplication.getType())
                .eq(PayBox::getStatus, StatusEnum.YES.getCode())
                .last("limit 1"));
        //修改游戏支付配置
        gamePayWayService.update(new LambdaUpdateWrapper<GamePayWay>()
                .in(GamePayWay::getPayBoxId, payBoxList)
                .set(GamePayWay::getPayBoxId, payBox.getId())
                .set(GamePayWay::getUpdateTime, LocalDateTime.now()));
    }

    private void sendDingTalkMsg(PayApplication payApplication) {
        List<ListenCall> listenCallList = this.callListenUser(payApplication);
        if (CollectionUtils.isEmpty(listenCallList)) {
            return;
        }
        String content = "小程序 : " + payApplication.getAppName() + " 可能被封停或者禁用, 请注意验证";
        Set<Long> userIdSet = listenCallList.stream().map(ListenCall::getUserId).collect(Collectors.toSet());
        userIdSet.forEach(userId -> dingTalkMsgRpc.sendByUserId(userId, content));
    }

    private void sendPhoneTalkMsg(PayApplication payApplication) {
        List<ListenCall> listenCallList = this.callListenUser(payApplication);
        if (CollectionUtils.isEmpty(listenCallList)) {
            return;
        }
        String content = payApplication.getAppName() + "监测异常";
        Set<String> phoneNumSet = listenCallList.stream().map(ListenCall::getPhoneNum).collect(Collectors.toSet());
        phoneNumSet.forEach(phoneNum -> this.phoneCall(phoneNum, content));
    }

    private List<ListenCall> callListenUser(PayApplication payApplication) {
        //支付应用被使用到的盒子
        List<Integer> payBoxIdList = payBoxService.list(new LambdaQueryWrapper<PayBox>()
                .eq(PayBox::getAppId, payApplication.getAppId())
        ).stream().map(PayBox::getId).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(payBoxIdList)) {
            return Collections.emptyList();
        }
        //盒子被使用到的游戏
        Set<Long> gameIdSet = gamePayWayService.list(new LambdaQueryWrapper<GamePayWay>()
                .in(GamePayWay::getPayBoxId, payBoxIdList)
        ).stream().map(GamePayWay::getGameId).collect(Collectors.toSet());
        if (CollectionUtils.isEmpty(gameIdSet)) {
            return Collections.emptyList();
        }
        //游戏关联的监听人员
        return listenCallService.list(new LambdaQueryWrapper<ListenCall>().in(ListenCall::getGameId, gameIdSet));
    }

    private boolean appletCheck(String token) {
        //参数对象
        String dayTime = LocalDate.now().minusDays(1).toString();
        Map<String, String> paramMap = new HashMap<>(2);
        paramMap.put("begin_date", dayTime);
        paramMap.put("end_date", dayTime);
        String host = "https://api.weixin.qq.com/datacube/getweanalysisappiddailyretaininfo?access_token=" + token;
        try {
            // 带参POST请求
            String result = HttpUtil.post(host, JsonUtil.toString(paramMap));
            if (Strings.isNotBlank(result) && result.contains("errcode")) {
                return Boolean.FALSE;
            }
        } catch (Exception e) {
            log.error("小程序token检测出现异常, token : {},  e : {}", token, e.getMessage());
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    private void phoneCall(String param, String mobile) {
        RestTemplate restTemplate = new RestTemplate();
        String appCode = "f395b1587fc04a49a975f908660fb1e9";
        String host = "https://jumfixed.market.alicloudapi.com/voice-notify/send";
        HttpHeaders headers = new HttpHeaders();
        headers.set("Authorization", "APPCODE " + appCode);
        headers.set("Content-Type", ContentType.FORM_URLENCODED.getValue());
        //参数
        MultiValueMap<String, String> requestParams = new LinkedMultiValueMap<>();
        requestParams.add("mobile", mobile);
        requestParams.add("templateId", "JMJNAWUQOJP9");
        requestParams.add("param", param);
        String result = null;
        try {
            ResponseEntity<String> responseEntity = restTemplate.exchange(host, HttpMethod.POST,
                    new HttpEntity<>(requestParams, headers), String.class);
            result = responseEntity.getBody();
        } catch (Exception e) {
            log.error("阿里语音呼叫失败, requestParams : {}, e : {}", JsonUtil.toString(requestParams), e.getMessage());
        }
        log.error("阿里语音呼叫结果, result : {}", result);
    }

}
